Package: CodelistGenerator
Title: Identify Relevant Clinical Codes and Evaluate Their Use
Version: 4.1.0
Authors@R: c(
    person("Edward", "Burn", email = "edward.burn@ndorms.ox.ac.uk", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-9286-1128")),
    person( "Marta", "Alcalde-Herraiz", email = "marta.alcaldeherraiz@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0009-0002-4405-1814")),
    person("Martí", "Català", email = "marti.catalasabate@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0003-3308-9905")),
    person("Xihang", "Chen", email = "xihang.chen@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0009-0001-8112-8959")),
    person("Nuria", "Mercade-Besora", email = "nuria.mercadebesora@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0006-7948-3747")),
    person("Mike", "Du", email = "mike.du@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-9517-8834")),
    person("Danielle", "Newby", email = "danielle.newby@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-3001-1478"))
    )
Description: Generate a candidate code list for the Observational Medical Outcomes Partnership (OMOP) common data model based on string matching. For a given search strategy, a candidate code list will be returned.
License: Apache License (>= 2)
Encoding: UTF-8
Depends: R (>= 4.1)
Imports: cli (>= 3.1.0), clock, DBI (>= 1.1.0), dplyr (>= 1.1.0), glue
        (>= 1.5.0), lifecycle, omopgenerics (>= 1.3.4), PatientProfiles
        (>= 1.4.4), purrr, rlang (>= 1.0.0), stringi (>= 1.8.1),
        stringr (>= 1.6.0), tidyr (>= 1.2.0), vctrs
Suggests: omock, covr, duckdb, CDMConnector (>= 2.1.0), visOmopResults
        (>= 1.4.0), CohortConstructor (>= 0.5.0), jsonlite, knitr,
        rmarkdown, testthat (>= 3.0.0), RPostgres, odbc, openxlsx,
        spelling, tibble, gt, flextable, tictoc
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
URL: https://darwin-eu.github.io/CodelistGenerator/
BugReports: https://github.com/darwin-eu/CodelistGenerator/issues
Language: en-US
LazyData: true
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-01 14:22:10 UTC; orms0426
Author: Edward Burn [aut, cre] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Marta Alcalde-Herraiz [aut] (ORCID:
    <https://orcid.org/0009-0002-4405-1814>),
  Martí Català [aut] (ORCID: <https://orcid.org/0000-0003-3308-9905>),
  Xihang Chen [aut] (ORCID: <https://orcid.org/0009-0001-8112-8959>),
  Nuria Mercade-Besora [aut] (ORCID:
    <https://orcid.org/0009-0006-7948-3747>),
  Mike Du [aut] (ORCID: <https://orcid.org/0000-0002-9517-8834>),
  Danielle Newby [aut] (ORCID: <https://orcid.org/0000-0002-3001-1478>)
Maintainer: Edward Burn <edward.burn@ndorms.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-08-01 15:10:02 UTC
