# Generated by roxygen2: do not edit by hand

S3method(asCodelist,candidate_codes)
S3method(asCodelist,code_search)
S3method(asCodelist,codelist)
S3method(asCodelist,codelist_with_details)
S3method(asCodelist,concept_set_expression)
S3method(asCodelistWithDetails,candidate_codes)
S3method(asCodelistWithDetails,code_search)
S3method(asCodelistWithDetails,codelist)
S3method(asCodelistWithDetails,codelist_with_details)
S3method(asConceptSetExpression,code_search)
S3method(asConceptSetExpression,codelist)
S3method(asConceptSetExpression,codelist_with_details)
S3method(print,code_search)
export(addConcepts)
export(asCodelist)
export(asCodelistWithDetails)
export(asConceptSetExpression)
export(associatedConceptClassIds)
export(associatedDomains)
export(associatedDoseForms)
export(associatedDoseUnits)
export(associatedDrugIngredients)
export(associatedRelationshipIds)
export(associatedRouteCategories)
export(associatedVocabularies)
export(availableATC)
export(availableConceptClassIds)
export(availableDomains)
export(availableDoseForms)
export(availableDoseUnits)
export(availableDrugIngredients)
export(availableRelationshipIds)
export(availableRouteCategories)
export(availableVocabularies)
export(benchmarkCodelistGenerator)
export(codesFromCohort)
export(codesFromConceptSet)
export(cohortCodelist)
export(compareCodelists)
export(emptyCodeSearch)
export(excludeConcepts)
export(exportCodeSearch)
export(exportCodelist)
export(exportConceptSetExpression)
export(getATCCodes)
export(getCandidateCodes)
export(getDescendants)
export(getDrugIngredientCodes)
export(getMappings)
export(importCodeSearch)
export(importCodelist)
export(importConceptSetExpression)
export(intersectCodelists)
export(mockVocabRef)
export(newCodelist)
export(newCodelistWithDetails)
export(newConceptSetExpression)
export(searchStrategy)
export(stratifyByBrand)
export(stratifyByConcept)
export(stratifyByDomain)
export(stratifyByDoseForm)
export(stratifyByDoseUnit)
export(stratifyByRouteCategory)
export(stratifyByVocabulary)
export(subsetOnDomain)
export(subsetOnDoseForm)
export(subsetOnDoseUnit)
export(subsetOnIngredientRange)
export(subsetOnRouteCategory)
export(subsetOnVocabulary)
export(subsetToCodesInUse)
export(summariseAchillesCodeUse)
export(summariseCodeUse)
export(summariseCohortCodeUse)
export(summariseOrphanCodes)
export(tableAchillesCodeUse)
export(tableCodeUse)
export(tableCohortCodeUse)
export(tableOrphanCodes)
export(unionCodelists)
export(vocabularyVersion)
importFrom(lifecycle,deprecated)
importFrom(omopgenerics,cohortCodelist)
importFrom(omopgenerics,exportCodelist)
importFrom(omopgenerics,exportConceptSetExpression)
importFrom(omopgenerics,importCodelist)
importFrom(omopgenerics,importConceptSetExpression)
importFrom(omopgenerics,newCodelist)
importFrom(omopgenerics,newCodelistWithDetails)
importFrom(omopgenerics,newConceptSetExpression)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,.env)
