#' @importFrom omopgenerics cohortCodelist
#' @export
omopgenerics::cohortCodelist

#' @importFrom omopgenerics importCodelist
#' @export
omopgenerics::importCodelist

#' @importFrom omopgenerics importConceptSetExpression
#' @export
omopgenerics::importConceptSetExpression

#' @importFrom omopgenerics exportCodelist
#' @export
omopgenerics::exportCodelist

#' @importFrom omopgenerics exportConceptSetExpression
#' @export
omopgenerics::exportConceptSetExpression

#' @importFrom omopgenerics newCodelist
#' @export
omopgenerics::newCodelist

#' @importFrom omopgenerics newCodelistWithDetails
#' @export
omopgenerics::newCodelistWithDetails

#' @importFrom omopgenerics newConceptSetExpression
#' @export
omopgenerics::newConceptSetExpression
