#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom rlang .data
#' @importFrom rlang .env
#' @importFrom rlang :=
## usethis namespace: end
NULL
