# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes from a particular domain.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams domainDoc
#' @param negate If FALSE, only concepts with the domain specified will be
#' returned. If TRUE, concepts with the domain specified will be excluded.
#'
#' @return The codelist with only those concepts associated with the domain
#' (if negate = FALSE) or the codelist without those concepts associated with
#' the domain (if negate = TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- subsetOnDomain(
#'               x = newCodelist(list("codes" = c(10,13,15))),
#'               cdm = cdm,
#'               domain = "Drug")
#' codes
#' }
subsetOnDomain <- function(x,
                           cdm,
                           domain,
                           negate = FALSE){

  omopgenerics::assertCharacter(domain, null = FALSE, na = FALSE)
  st <- searchStrategyAttr(
    function_name = "subsetOnDomain",
    cdm = "cdm",
    domain = cast(domain)
  )
  x <- subsetCodelistBy(x,
                        cdm,
                        by = "domain",
                        group = domain,
                        keepOriginal = FALSE,
                        st = st,
                        negate = negate)
  return(x)
}
